unit knots;

interface

uses
  Geometry, dglOpenGl;

const
  KNOT_SEGMENTS = 500;
  TUBE_RADIUS   = 1.0 / 5.0;
  SEGMENT_SLICES = 32;

procedure RenderKnotPoints;
procedure RenderTubePoints;
procedure RenderTube;

function NumVerts : Integer;
function NumTriangles : Integer;

implementation

var
  line_points : Array[0..KNOT_SEGMENTS] of TVertex;
  knot_vertices : Array[0..KNOT_SEGMENTS] of Array[0..SEGMENT_SLICES] of TVertex;
  knot_normals : Array[0..KNOT_SEGMENTS] of Array[0..SEGMENT_SLICES] of TVertex;

function NumVerts : Integer;
begin
  Result := (KNOT_SEGMENTS + 1) * (SEGMENT_SLICES + 1);
end;

function NumTriangles : Integer;
begin
  Result := KNOT_SEGMENTS * (SEGMENT_SLICES * 2);
end;

procedure RenderKnotPoints;
var
  i : Integer;
begin
  glBegin(GL_POINTS);
    for i := KNOT_SEGMENTS -1 downto 0 do
    begin
      glVertex3fv(@line_points[i][0]);
    end;
  glEnd();
end;

procedure RenderTubePoints;
var
  i,j : Integer;
begin
  glBegin(GL_POINTS);
    for i := KNOT_SEGMENTS -1 downto 0 do
      for j := SEGMENT_SLICES -1 downto 0 do
      begin
        glVertex3fv(@knot_vertices[i][j][0]);
      end;
  glEnd();
end;

procedure RenderTube;
var
  i,j : Integer;
begin
  for i := 0 to KNOT_SEGMENTS - 1 do
  begin
    glBegin(GL_TRIANGLE_STRIP);
      for j := 0 to SEGMENT_SLICES do
      begin
        glNormal3fv(@knot_normals[i][j][0]);
        glVertex3fv(@knot_vertices[i][j][0]);

        glNormal3fv(@knot_normals[i+1][j][0]);
        glVertex3fv(@knot_vertices[i+1][j][0]);
      end;
    glEnd();
  end;
end;

procedure CreateKnotData;
var
  i, j : Integer;
  mu : Extended;
  Basis : Array[0..2] of TVertex;
begin
  for i := 0 to KNOT_SEGMENTS do
  begin
    mu := 1 / KNOT_SEGMENTS * 2 * Pi * i;
    //f(x)
    line_points[i][0] := (4 * cos(mu + Pi)) / 3 + 2 * cos(3 * mu);
    line_points[i][1] := 4 * sin(mu) / 3 + 2 * sin(3 * mu);
    line_points[i][2] := sin(4 * mu) + sin(2 * mu) / 2;

    //f'(x)
    Basis[0][0] := -4.0/3.0 * sin(mu + pi) - 6 * sin(3 * mu);
    Basis[0][1] := 4.0/3.0  * cos(mu     ) + 6 * cos(3 * mu);
    Basis[0][2] := 4.0      * cos(4 * mu ) +     cos(2 * mu);
    NormalizeVector(Basis[0]);

    //f''(x)
    Basis[1][0] := -4.0/3.0 * cos(mu + pi) - 18 * cos(3 * mu);
    Basis[1][1] := -4.0/3.0 * sin(mu     ) - 18 * sin(3 * mu);
    Basis[1][2] := - 16.0   * sin(4 * mu ) - 2  * sin(2 * mu);
    NormalizeVector(Basis[1]);

    //Mitlaufendes Dreibein
    Basis[2] := VectorCrossProduct(Basis[0], Basis[1]);

    for j := 0 to SEGMENT_SLICES do
    begin
      knot_normals[i][j] := VectorAdd(
        VectorScale(Basis[1], sin(1/SEGMENT_SLICES * 2* Pi * j)),
        VectorScale(Basis[2], cos(1/SEGMENT_SLICES * 2* Pi * j)));

       knot_vertices[i][j] := VectorAdd(VectorScale(knot_normals[i][j], TUBE_RADIUS),
         line_points[i]);
    end;
  end;
end;

initialization
  CreateKnotData;

end.
